/*
 * Copyright (C) 2014 Yahoo Inc.
 */

// A constant prefix which is part of every attachment id
var ATTACHMENT_ID_PREFIX = "ymail_attachmentId";

// The class name for storing the content id of an attachment
var INLINE_ATTACHMENT_CONTENT_ID_CLASS_NAME_PREFIX = "inlined-image-cid-";

// A place holder token representing the attachment image data or the image url
var ATTACHMENT_ID_TOKEN = "#ID#";
var ATTACHMENT_URL_TOKEN = "#DATA_URL#";
var ATTACHMENT_PLACEHOLDER_URL_TOKEN = "#PLACEHOLDER_URL#";
var ATTACHMENT_CONTENT_ID_TOKEN = "#CONTENT_ID#";

// Various style class names used to style and look up elements
// These might be used in other files, make sure any name changes are reflected elsewhere
var INLINED_ATTACHMENT_CLASS_NAME = "inlined-image";
var ATTACHMENT_IMAGE_ELEMENT_CLASS = "attachment-image";
var ATTACHMENT_PLACEHOLDER_ELEMENT_CLASS = "attachment-placeholder";

// A place holder token representing the attachment name
var ATTACHMENT_NAME_TOKEN = "#ATT_NAME#";

// A place holder token representing the attachment size
var ATTACHMENT_SIZE_TOKEN = "#ATT_SIZE#";

// Each attachment is represented by this template
var ATTACHMENT_TEMPLATE = "<img src=\"./images/attachments_delete_button.png\" class=\"attachment-delete-btn\">" +
        "<div class=\"" + ATTACHMENT_PLACEHOLDER_ELEMENT_CLASS + "\"><div class=\"center\"></div><img src=\"" + ATTACHMENT_PLACEHOLDER_URL_TOKEN + "\"/></div>" +
        "<div class=\"" + ATTACHMENT_IMAGE_ELEMENT_CLASS + "\" style=\"background-image: url(\'" + ATTACHMENT_URL_TOKEN +"\');\"></div>" +
        "<div class=\"attachment-name-label default-text-style\">" + ATTACHMENT_NAME_TOKEN + "</div>" +
        "<div class=\"attachment-size-label default-text-style\">" + ATTACHMENT_SIZE_TOKEN + "</div>";

// Each inline image attachment is represented by this template
var INLINE_ATTACHMENT_TEMPLATE = "<img id=\"" + ATTACHMENT_ID_TOKEN + "\" src=\"" + ATTACHMENT_URL_TOKEN + "\"" + " style=\"width:100%;max-width:800px;\" class=\"" + INLINED_ATTACHMENT_CLASS_NAME + " " + INLINE_ATTACHMENT_CONTENT_ID_CLASS_NAME_PREFIX + ATTACHMENT_CONTENT_ID_TOKEN + "\"><br>";

/**
 * An object which controls the attachment band, including adding attachments into view and showing/hiding the detailed view.
 */
var attachmentsBandController = {
    attachmentsContainerNode: null,
    attachmentsNode: null,
    attachmentsLabelNode: null,
    composeContentNode: null,
    numAttachments: 0,
    totalSizeBytes: 0,
    inlineAttachmentNoCacheQueryParam: null,

    /**
     * A method which starts the attachmentsBandController
     */
    init: function(attachmentsContainerNode, attachmentsNode, attachmentsLabelNode, composeContentNode, inlineAttachmentNoCacheQueryParam) {
        this.attachmentsContainerNode = attachmentsContainerNode;
        this.attachmentsNode = attachmentsNode;
        this.attachmentsLabelNode = attachmentsLabelNode;
        this.composeContentNode = composeContentNode;
        this.inlineAttachmentNoCacheQueryParam = inlineAttachmentNoCacheQueryParam;
    },

    /**
     * Add an attachment with the given parameters. Usually called by native code.
     */
    addAttachment: function(id, name, size, imagePlaceHolder, imageUri, sizeString) {
        var attachmentNode = document.getElementById(ATTACHMENT_ID_PREFIX + id);

        if (!attachmentNode) {
            // New attachment, so create a node and configure it
            attachmentNode = document.createElement("li");
            attachmentNode.setAttribute("class", "attachment");
            attachmentNode.setAttribute("id", ATTACHMENT_ID_PREFIX + id);
            attachmentNode.addEventListener("click", this.attachmentClickHandler.bind(this), false);
            this.numAttachments++;
        } else {
            this.totalSizeBytes -= parseInt(attachmentNode.getAttribute("attachment-size"), 10);
        }

        attachmentNode.innerHTML = ATTACHMENT_TEMPLATE
                .replace(ATTACHMENT_URL_TOKEN, imageUri)
                .replace(ATTACHMENT_NAME_TOKEN, name)
                .replace(ATTACHMENT_SIZE_TOKEN, sizeString)
                .replace(ATTACHMENT_PLACEHOLDER_URL_TOKEN, imagePlaceHolder);

        var image = new Image();
        image.addEventListener("load", this.attachmentImageLoadHandler.bind(this, attachmentNode), true);
        image.setAttribute("src", imageUri);

        if (this.numAttachments > 0) {
            this.attachmentsContainerNode.classList.remove("hidden");
        }

        // Update the size
        this.totalSizeBytes += size;

        attachmentNode.setAttribute("attachment-size", size);

        this.attachmentsNode.appendChild(attachmentNode);

        this.attachmentsLabelNode.textContent = yMailBridge.getAttachmentBandString(this.numAttachments, this.totalSizeBytes);

        attachmentNode.lastChild.setAttribute("aria-label", name);
    },

    /**
     * Adds an inline attachment at the current cursor position
     */
    addInlineAttachment: function(id, imagePlaceHolder, imageUri, contentId) {
        // Prevent browser from aggressively caching modified inline image
        // Query param will be removed before sending
        var noCacheUri = imageUri + '?' + this.inlineAttachmentNoCacheQueryParam;

        // Make sure the focus is in the compose message body
        this.composeContentNode.focus();

        // Insert the HTML Template representing an inline attachment
        document.execCommand("insertHTML", false, INLINE_ATTACHMENT_TEMPLATE.replace(ATTACHMENT_ID_TOKEN, ATTACHMENT_ID_PREFIX + id).replace(ATTACHMENT_URL_TOKEN, noCacheUri).replace(ATTACHMENT_CONTENT_ID_TOKEN, contentId));

        // Call the auto save controller to immediately save the message body, so that the message has a reference to the inlined attachment
        autoSaveController.handleInput(null);
    },

    /**
     * Remove an attachment with the given id
     */
    removeAttachment: function(id) {
        var attachmentNode = document.getElementById(ATTACHMENT_ID_PREFIX + id);

        if (attachmentNode) {
            attachmentNode.parentNode.removeChild(attachmentNode);

            this.totalSizeBytes -= parseInt(attachmentNode.getAttribute("attachment-size"), 10);

            this.numAttachments--;

            this.attachmentsLabelNode.textContent = yMailBridge.getAttachmentBandString(this.numAttachments, this.totalSizeBytes);

            if (this.numAttachments === 0) {
                this.attachmentsContainerNode.classList.add("hidden");
            }
        }
    },

    /**
     * Click handler for each attachment
     */
    attachmentClickHandler: function(event) {
        yMailBridge.showAttachmentClickOptions(event.currentTarget.id.replace(ATTACHMENT_ID_PREFIX, ""));
    },

    /**
     * This method is used to handle errors while loading attachment preview images
     */
    attachmentImageLoadHandler: function(attachmentNode, event) {
        attachmentNode.getElementsByClassName(ATTACHMENT_IMAGE_ELEMENT_CLASS)[0].style.visibility = "visible";
        attachmentNode.getElementsByClassName(ATTACHMENT_PLACEHOLDER_ELEMENT_CLASS)[0].style.visibility = "hidden";
    }
};
